Package: customCMPdb
Title: Customize and Query Compound Annotation Database
Description: This package serves as a query interface for important community collections of
    small molecules, while also allowing users to include custom compound collections. 
Version: 1.23.0
Authors@R: 
    c(person("Yuzhu", "Duan", role = c("aut", "cre"), email = "yduan004@ucr.edu"),
      person("Thomas", "Girke", role = "aut", email = "thomas.girke@ucr.edu"))
Depends: R (>= 4.0)
Imports: AnnotationHub, RSQLite, XML, utils, ChemmineR, methods, stats,
        rappdirs, BiocFileCache
Suggests: knitr, rmarkdown, testthat, BiocStyle
License: Artistic-2.0
biocViews: Software, Cheminformatics,AnnotationHubSoftware
Encoding: UTF-8
VignetteBuilder: knitr
URL: https://github.com/yduan004/customCMPdb/
BugReports: https://github.com/yduan004/customCMPdb/issues
RoxygenNote: 7.1.2
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/customCMPdb
git_branch: devel
git_last_commit: 055aeb1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:07:26 UTC; biocbuild
Author: Yuzhu Duan [aut, cre],
  Thomas Girke [aut]
Maintainer: Yuzhu Duan <yduan004@ucr.edu>

# Generated by roxygen2: do not edit by hand

export(addCustomAnnot)
export(buildDrugAgeDB)
export(dbxml2df)
export(defaultAnnot)
export(deleteAnnot)
export(df2SQLite)
export(listAnnot)
export(loadAnnot)
export(loadSDFwithName)
export(processDrugage)
export(queryAnnotDB)
import(AnnotationHub)
import(BiocFileCache)
import(ChemmineR)
import(RSQLite)
importFrom(AnnotationHub,AnnotationHub)
importFrom(XML,xmlParse)
importFrom(XML,xmlRoot)
importFrom(XML,xmlSize)
importFrom(XML,xmlToDataFrame)
importFrom(methods,as)
importFrom(rappdirs,user_cache_dir)
importFrom(stats,na.omit)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)

# customCMPdb: Integrating Community and Custom Compound Collections

## Introduction
This package serves as a query interface for important community collections of
small molecules, while also allowing users to include custom compound
collections. At the time of writing, the following community databases are included: 

+ [DrugAge](https://genomics.senescence.info/drugs/) 
+ [DrugBank](https://www.drugbank.ca/) 
+ [CMAP02](https://portals.broadinstitute.org/cmap/) 
+ [LINCS](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742)


## Installation and Loading
`customCMPdb` is an R/Bioconductor package and can be installed using 
`BiocManager::install()`.
```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("customCMPdb")
```

To obtain the most recent updates immediately, one can install it directly from 
GitHub as follows.
```r
devtools::install_github("yduan004/customCMPdb", build_vignettes=TRUE)
```

After the package is installed, it can be loaded into an R session as follows.
```r
library(customCMPdb)
```
For detailed description of the package, please refer to the vignette by running
```r
browseVignettes("customCMPdb")
```

## Vignette
The vignette of this package is also available at [here](https://www.bioconductor.org/packages/release/bioc/vignettes/customCMPdb/inst/doc/customCMPdb.html)

###################################################
## SQLite Structure Database for Drugs from CMAP ##
###################################################
## Author: Thomas Girke
## Last update: 13-May-16

## Obtaining the structures and PubChem IDs for CMAP was much harder than expected since 
## cmap only provides inconsistently formatted compound names and order numbers. The following
## documents the workflow you used.

#' Build CMAP Database
#' 
#' This function builds a SQLite database named as 'cmap.db' that contains id 
#' mappings of cmap names to PubChem/DrugBank IDs as well as compound structure information.
#' 
#' For about 2/3 of the CMAP drugs, one can obtain their PubChem/DrugBank IDs from 
#' the DMAP site here: http://bio.informatics.iupui.edu/cmaps. Since this website is no
#' longer supported, the processed CMAP name to PubChem and DrugBank ID mapping table
#' is stored under the "inst/extdata" folder of this package named as "dmap_unique.txt".  
#' The SMILES strings for CMAP entries were obtained from ChemBank. Compounds 
#' were matched by names using the 'stringdist' library where cmap_name from 
#' CMAP were mapped to the closest name in ChemBank.
#' @param dest_dir character(1), destination directory under which the result 
#' SQLite database named as 'cmap.db' stored. The default is user's current
#' working directory.
#' @return write "cmap.db" SQLite database to the destination directory defined by user.
#' @import ChemmineR
#' @importFrom utils download.file
#' @importFrom utils read.delim
#' @importFrom utils write.table
#' @importFrom methods as
#' @importFrom stats na.omit
#' @examples 
#' library(ChemmineR)
#' ## Query database
#' # buildCMAPdb(dest_dir="./inst/scripts")
#' # conn <- initDb("/inst/scripts/cmap.db")
#' # results <- getAllCompoundIds(conn)
#' # sdfset <- getCompounds(conn, results, keepOrder=TRUE)
#' # sdfset
#' # as.data.frame(datablock2ma(datablock(sdfset)))[1:4,]
#' # myfeat <- listFeatures(conn)
#' # feat <- getCompoundFeatures(conn, results, myfeat)
#' # feat[1:4,]
buildCMAPdb <- function(dest_dir=".") {
        ## Join DMAP and CMAP tables
        dmap_path <- system.file("extdata/dmap_unique.txt", package="customCMPdb")
        dmap <- read.delim(dmap_path)
        row.names(dmap) <- tolower(dmap$SOURCE_DRUG)
        
        cmap_inst <- system.file("extdata/cmap_instances_02.txt", package="customCMPdb")
        cmap <- read.delim(cmap_inst)
        cmap <- cmap[!duplicated(tolower(cmap$cmap_name)),]
        row.names(cmap) <- tolower(cmap$cmap_name)
        df <- data.frame(cmap, dmap[row.names(cmap),])
        # sum(!is.na(df$SOURCE_DRUG))
        # length(unique(na.omit(df$SOURCE_DRUG)))
        # 867 cmap drugs have PubChem IDs; 442 do not have; 
        # instance_id 1345 and 2952 with two cmap_name "betulinic acid" and "betulin"
        # but the same SOURCE_DRUG "Betulinic Acid"

        ## Obtain SMILES strings for CMAP entries from ChemBank
        ## Tyler did this with help from P. Clemens from ChemBank.
        ## Compounds were matched by names using the stringdist library
        ## (here cmap_name from CMAP and the closest name in ChemBank).
        ## This is the location of the input files:
        ## /rhome/tbackman/Projects/cmap_drugs/src/mapIDs.R
        ## /rhome/tbackman/Projects/cmap_drugs/working/cmap_instances_02.xls
        ## /rhome/tbackman/Projects/cmap_drugs/working/allcompounds.smiles
        ## /rhome/tbackman/Projects/cmap_drugs/working/chembank-synonyms.txt
        ## /rhome/tbackman/Projects/cmap_drugs/working/chembank-structures.txt
        ## /rhome/tbackman/Projects/cmap_drugs/working/smilesMatches.tsv

        ## Note: file smilesMatches.tsv was generated by Tyler, see above
        smipath <- system.file("extdata/smilesMatches.tsv", package="customCMPdb")
        smiMA <- read.csv(smipath) 
        row.names(smiMA) <- tolower(smiMA$cmap_name)
        bothDF <- cbind(df, smiMA[rownames(df),
                c("chembank_id", "chembank_name", "match_distance", "smiles")])
        #dim(bothDF[as.integer(bothDF[,"match_distance"]) == 0,]) # 1223 x 28
        # Note: compounds with match_distance > 0 need to be checked

        ## Create SDFset 
        #library(ChemmineR); library(ChemmineOB) # requires openbabel module
        smi <- as.character(bothDF$smiles)
        names(smi) <- as.character(bothDF$cmap_name)
        smi <- as(smi, "SMIset")
        sdfset <- smiles2sdf(smi)
        datablock(sdfset) <- bothDF # Stores annotation info in datablock slots

        ## Create SQLite database
        standardFeatures <- function(sdfInput) {
            data.frame(propOB(sdfInput),
                Ncharges=sapply(bonds(sdfInput, type="charge"), length),
                as.data.frame(groups(sdfInput, type="countMA")),
                as.data.frame(rings(sdfInput, upper=8, type="count", arom=TRUE)))
        }
        conn <- initDb(paste0(dest_dir, "/cmap.db"))
        ids <- loadSdf(conn, sdfset, fct=standardFeatures)
        dbDisconnect(conn)
}

#' Build DrugAge Annotation Database
#' 
#' This function builds the DrugAge annotation SQLite database from the 
#' 'drugage_id_mapping' table stored in the 'inst/extdata' directory of this 
#' package. The 'drugage_id_mapping.tsv' table contains the DrugAge compounds 
#' annotation information (such as species, avg_lifespan_change etc) as well 
#' as the compound name to ChEMBL id and PubChem id mappings. 
#' 
#' Part of the id mappings in the 'drugage_id_mapping.tsv' table is generated 
#' by the \code{processDrugage} function for compound names that have ChEMBL 
#' ids from the ChEMBL database (version 24). The missing IDs were added 
#' manually. A semi-manual approach was to use this web service: 
#' \url{https://cts.fiehnlab.ucdavis.edu/batch}. After the semi-manual process,
#' the left ones were manually mapped to ChEMBL, PubChem and DrugBank ids. 
#' The mixed items were commented. 
#' @param da_path character(1), file path to the tabular file generated from
#' \code{processDrugage} function and manually edited the missing IDs.
#' @param dest_path character(1), destination path of the result DrugAge 
#' annotation SQLite database
#' @return DrugAge annotation SQLite database 
#' @examples 
#' buildDrugAgeDB(dest_path=tempfile(fileext="_drugage.db"))
#' @export
#'  
buildDrugAgeDB <- function(da_path=system.file("extdata/drugage_id_mapping.tsv", 
                                    package="customCMPdb"), dest_path){
    ## Read DrugAge Mapping from inst/extdata
    
    # require(gsheet)
    # url<-"https://bit.ly/3dCWKWo"
    # drugAge_mapping <- suppressWarnings(gsheet2tbl(url))
    # ## Add columns names
    # names(drugAge_mapping) <- as.character(unlist(drugAge_mapping[1,]))
    # drugAge_mapping <- drugAge_mapping[-1,]
    
    drugAge_mapping <- read.delim(da_path)
    ## Create internal DrugAge_id named ida000xxx
    drugAge_ids <- paste0("ida",sprintf("%05d",1:nrow(drugAge_mapping)))
    drugAge_mapping2 <- data.frame(drugage_id=drugAge_ids, drugAge_mapping)
    
    ## Create DrugAge Database
    da <- dbConnect(SQLite(), dest_path)
    ## write id_mapping table.
    id_maptb <- na.omit(data.frame(drugage_id=drugAge_mapping2$drugage_id,
                                   chembl_id=drugAge_mapping2$chembl_id, 
                                   stringsAsFactors=FALSE)) 
    dbWriteTable(da,'id_mapping', id_maptb) 
    
    ## write DrugAge Annotation table
    drugAgeAnnot <- data.frame(drugAge_mapping2[1:13], drugAge_mapping2[15:16])
    dbWriteTable(da, 'drugAgeAnnot', drugAgeAnnot)
    dbDisconnect(da)
}

#' Process Source DrugAge Dataset
#' 
#' This function processes the source DrugAge datasets by adding the ChEMBL, 
#' PubChem and DrugBank id mapping information to the source DrugAge table 
#' which only has compound names without id mapping information. Source file of 
#' DrugAge is linked here: \url{http://genomics.senescence.info/drugs/dataset.zip}
#' 
#' This function only annotates compound names that have ChEMBL 
#' ids from the ChEMBL database (version 24). The missing IDs were added 
#' manually. A semi-manual approach was to use this web service: 
#' \url{https://cts.fiehnlab.ucdavis.edu/batch}. After the semi-manual process,
#' the left ones were manually mapped to ChEMBL, PubChem and DrugBank ids. 
#' The mixed items were commented. 
#' @param dest_file character(1), file path to the generated DrugAge annotation 
#' tabular file with id mappings. The default will write the file named as
#' "drugage_id_mapping.tsv" to user's current working directory.
#' @param verbose logical(1), If descriptive message and list of issues should be 
#' included as output
#' @return write the default 'drugage_id_mapping.tsv' file to user's current working
#' directory or the file path defined by users to the \code{dest_dafile} argument.
#' @importFrom rappdirs user_cache_dir
#' @importFrom utils untar
#' @import BiocFileCache
#' @examples 
#' library(ChemmineR)
#' \dontrun{
#' processDrugage(dest_file="drugage_id_mapping.tsv")
#' # Now the missing IDs need to be added manually. A semi-manual approach is to 
#' # use this web service: https://cts.fiehnlab.ucdavis.edu/batch
#' }
#' @export
#' 
processDrugage <- function(dest_file="drugage_id_mapping.tsv", verbose=TRUE) {
    # download DrugAge dataset to Cache directory
    dazip <- download_data("http://genomics.senescence.info/drugs/dataset.zip", 
                            "DrugAge_dataset_zip", verbose=verbose)
    ## read tsv file in the zip by temporally extracting zip to tempfile
    tmpdir <- tempdir()
    unzip(dazip, exdir=tmpdir)
    drugage <- read.csv(paste0(tmpdir, "/drugage.csv"))
    drugage <- drugage[!duplicated(drugage$compound_name),]
    
    ## Get mol_dict table from chembl_db
    chembldb <- download_data("ftp://ftp.ebi.ac.uk/pub/databases/chembl/ChEMBLdb/releases/chembl_24/archived/chembl_24_sqlite.tar.gz", 
                           "ChEMBL24", verbose=verbose, ftpUpdate=FALSE)
    bfc <- BiocFileCache(cache=rappdirs::user_cache_dir(appname="customCMPdb"), ask=TRUE)
    cache_dir <- bfccache(bfc)
    if(!dir.exists(paste0(cache_dir, "/chembl_24")))
        untar(chembldb, exdir = cache_dir)
    mydb <- dbConnect(SQLite(), paste0(cache_dir, "/chembl_24/chembl_24_sqlite/chembl_24.db"))
    mol_dict <- dbGetQuery(mydb, 'SELECT * FROM molecule_dictionary')
    mol_dict_sub <- mol_dict[!is.na(mol_dict$pref_name),] # mol_dict from below
    prefname <- as.character(mol_dict_sub$pref_name)
    chemblid <- as.character(mol_dict_sub$chembl_id)
    fact <- tapply(chemblid, factor(prefname), paste, collapse=", ")
    
    # Load ChEMBL to PubChem CID and DrugBank ID mappings
    ## UniChem CMP ID mappings from here: 
    ##      https://www.ebi.ac.uk/unichem/ucquery/listSources
    ## download ChEMBL to DrugBank mapping from UniChem in src1src2.txt.gz: 
    chembl2db <- download_data("ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src2.txt.gz", 
                               "chembl2drugbank", verbose = verbose)
    ## ChEMBL to PubChem CID mapping in src1src22.txt.gz
    chembl2pc <- download_data("ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src22.txt.gz", 
                                "chembl2pubchem", verbose = verbose)
    
    chembl2pubchem <- read.delim(gzfile(chembl2pc))
    chembl2pubchem_vec <- tapply(as.character(chembl2pubchem[,2]), 
                                 factor(chembl2pubchem[,1]), paste, collapse=", ")
    chembl2drugbank <- read.delim(gzfile(chembl2db))
    chembl2drugbank_vec <- tapply(as.character(chembl2drugbank[,2]), 
                                  factor(chembl2drugbank[,1]), paste, collapse=", ")
    ## Assemble results
    drugage <- cbind(drugage, pref_name=names(fact[toupper(drugage$compound_name)]), 
                     chembl_id=fact[toupper(drugage$compound_name)])
    drugage <- cbind(drugage, pubchem_cid=chembl2pubchem_vec[as.character(drugage$chembl_id)], 
                     DrugBank_id=chembl2drugbank_vec[as.character(drugage$chembl_id)])
    write.table(drugage, dest_file, row.names=FALSE, quote=FALSE, sep="\t")
}

download_data <- function(url, rname, verbose=TRUE, ftpUpdate=TRUE){
    bfc <- BiocFileCache(cache=rappdirs::user_cache_dir(appname="customCMPdb"), ask=TRUE)
    rid <- bfcquery(bfc, url)$rid
    if(!length(rid)){
        if(verbose) message(paste("Downloading", rname, "from", url))
        rid <- names(bfcadd(bfc, rname, url))
    }
    # check to see if the resource needs to be updated
    if (isTRUE(bfcneedsupdate(bfc, rid)) | ftpUpdate)
        bfcdownload(bfc, rid, ask=TRUE)
    bfcrpath(bfc, rids = rid)
}

##########################################
## Create SQLite database and SDF files ##
##########################################
## Author: Yuzhu Duan
## Last update: 12-April-2020

# The SQLite database containing id mappings and drug annotation tables in
# DrugAge, DrugBank, CMAP2 and LINCS databases
getwd() # get user's current working directory. In my case, named as "compoundCollection"

##############
## DrugBank ##
##############

# get drugbank annotation table
## go to https://www.drugbank.ca/releases/latest download the drugbank xml 
## database as 'drugbank_5.1.5.xml.zip' file to `inst/scripts` directory
unzip("inst/scripts/drugbank_5.1.5.xml.zip", exdir="inst/scripts")
file.rename("inst/scripts/full database.xml", "inst/scripts/drugbank_5.1.5.xml")
library(customCMPdb)
drugbank_dataframe <- dbxml2df(xmlfile="inst/scripts/drugbank_5.1.5.xml", version="5.1.5")
df2SQLite(dbdf=drugbank_dataframe, version="5.1.5", dest_dir="./inst/scripts")
library(RSQLite)
dbpath <- "inst/scripts/drugbank_5.1.5.db"
conn <- dbConnect(SQLite(), dbpath)
dbListTables(conn)
dbdf <- dbGetQuery(conn, 'SELECT * FROM dbdf')
pryr::object_size(dbdf)
colnames(dbdf)

# get drugbank id to ChEMBL id mapping table
download.file("ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src2.txt.gz",
              "inst/scripts/chembl2drugbank.txt.gz")
library(readr)
chem2db <- read_tsv("inst/scripts/chembl2drugbank.txt.gz") # 7290 X 2
colnames(chem2db) <- c("chembl_id", "drugbank_id")
length(intersect(dbdf$`drugbank-id`, chem2db$drugbank_id))
## 7290 / 13475 (54.1%) drugs in drugbank have chembl id
library(dplyr)
dbdf <- dbdf %>% rename("drugbank_id"="drugbank-id")

# download drugbank sdf file
## downlaod 'drugbank_all_structures.sdf.zip' file from 
## https://www.drugbank.ca/releases/latest#structures
## to the 'inst/scripts' directory
unzip("inst/scripts/drugbank_all_structures.sdf.zip", exdir="inst/scripts")
file.rename("inst/scripts/structures.sdf", "inst/scripts/drugbank_5.1.5.sdf")
db_stru <- read.SDFset("inst/scripts/drugbank_5.1.5.sdf") # 10,695

# Try to write drugbank id to Molecule_Name
dbids <- sapply(seq_along(db_stru), function(i){
    unlist(datablock(db_stru[[i]]))[1]
})
cid(db_stru) <- dbids
db_valid <- db_stru[validSDF(db_stru)] # 10,569
write.SDF(db_valid, "inst/scripts/drugbank_5.1.5.sdf", cid=TRUE)

#############
## DrugAge ##
#############

# Generate drugage.db by using 'buildDrugAgeDB' function
buildDrugAgeDB(dest_path="inst/scripts/drugage.db")
# Rename 'drugage.db' as 'compoundCollection.db' and add DrugBank compound 
# annotation and id_mappings to here
file.rename("inst/scripts/drugage.db", "inst/scripts/compoundCollection.db")
cc_path <- "inst/scripts/compoundCollection.db"
cc_conn <- dbConnect(SQLite(), cc_path)
dbListTables(cc_conn)
id_mapping <- dbGetQuery(cc_conn, "SELECT * FROM id_mapping")
## the 'chembl_id' column in DrugAge id mapping table contains several ChEMBL ids 
## in one row separated by comma
chem_vec <- id_mapping$chembl_id
names(chem_vec) <- id_mapping$drugage_id
chem_list <- lapply(chem_vec, function(x) unlist(strsplit(x, split=",")))
ida_dup <- rep(names(chem_list), times=sapply(chem_list, length))
id_mapping2 <- unique(data.frame(drugage_id=ida_dup, chembl_id=unlist(chem_list)))
# Add drugbank id to id mapping table
id_mapping3 <- merge(chem2db, id_mapping2, by="chembl_id", all.x=TRUE, all.y=TRUE) # 7448 X 3
dbWriteTable(cc_conn, "id_mapping", id_mapping3, overwrite=TRUE)
dbWriteTable(cc_conn, "DrugBankAnnot", dbdf)
dbListTables(cc_conn)
dbDisconnect(cc_conn)

# Get DrugAge sdf file
da_annot <- dbGetQuery(cc_conn, "SELECT * FROM drugAgeAnnot")
ida2pubchem <- na.omit(da_annot[,c("drugage_id", "pubchem_cid")])
pid_unique <- gsub(",.*", "", ida2pubchem$pubchem_cid)
library(ChemmineR)
da_cmp <- getIds(as.numeric(pid_unique))
cid(da_cmp) <- ida2pubchem$drugage_id
write.SDF(da_cmp, "inst/scripts/drugage.sdf", cid=TRUE)
## double check that the cids of the imported sdfset are drugage internal ids
da_cmp2 <- read.SDFset("inst/scripts/drugage.sdf")
head(cid(da_cmp2))
head(sdfid(da_cmp2))

###########
## CMAP2 ##
###########

# Get CMAP instance annotation table
download.file("https://portals.broadinstitute.org/cmap/cmap_instances_02.xls",
              "inst/scripts/cmap_instances_02.xls")
## Note, this file required some cleaning in LibreOffice (Excel would work for this too).
## The cleaning process removes the last lines that contain cell line info.
## After this it was saved as tab delimited txt file named 'cmap_instances_02.txt'
inst_path <- system.file("extdata/cmap_instances_02.txt", package="customCMPdb")
cmap_inst <- read.delim(inst_path, check.names=FALSE)
## The useful information in this cmap instance table are only compound names, 
## catelog number and catelog name

# Build 'cmap.db' 
## Since cmap only provides inconsistently formatted compound names and order 
## numbers. Obtaining the structures and PubChem IDs for CMAP was much harder 
## than expected. The 'buildCMAPdb' function builds 'cmap.db' that contains 
## id mapping and compound structure information.

## For about 2/3 of the CMAP drugs, one can obtain their PubChem/DrugBank IDs from 
## the DMAP site here: http://bio.informatics.iupui.edu/cmaps. 
## The SMILES strings for CMAP entries were obtained from ChemBank. Compounds 
## were matched by names using the 'stringdist' library where cmap_name from 
## CMAP were mapped to the closest name in ChemBank.
library(ChemmineR)
buildCMAPdb(dest_dir="inst/scripts")
conn <- dbConnect(SQLite(), "inst/scripts/cmap.db") # or conn <- initDb(mypath)
results <- getAllCompoundIds(conn)
sdfset <- getCompounds(conn, results, keepOrder=TRUE)
sdfset # An instance of "SDFset" with 1309 molecules
plot(sdfset[1:4], print=FALSE)

myfeat <- listFeatures(conn)
feat <- getCompoundFeatures(conn, results, myfeat)
feat[1:4,]

cmp_tb <- dbGetQuery(conn, 'SELECT * FROM compounds')
cmapAnnot <- as.data.frame(datablock2ma(datablock(sdfset))) # 1309 X 27
cmap_id <- paste0("CMAP", sprintf("%04d", as.integer(rownames(cmapAnnot))))
cmapAnnot2 <- data.frame(cmap_id=cmap_id, cmapAnnot, stringsAsFactors=FALSE)
# write CMAP SDF file
cid(sdfset) <- cmap_id
write.SDF(sdfset, file="inst/scripts/cmap.sdf", cid=TRUE)
cmap_sdf <- read.SDFset("inst/scripts/cmap.sdf")

# get pubchem cid to ChEMBL id mapping table
download.file("ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src22.txt.gz",
              "inst/scripts/chembl2pubchemcid.txt.gz")
library(readr)
chem2pub <- read_tsv("inst/scripts/chembl2pubchemcid.txt.gz") # 1837162 X 2
colnames(chem2pub) <- c("chembl_id", "pubchem_cid")
chem2pub$pubchem_cid <- as.character(chem2pub$pubchem_cid)
# get cmap_id (created internal) to pubchem cid mappings
cmap2pub <- cmapAnnot2[,c("cmap_id", "PUBCHEM_ID")]
sum(cmap2pub$PUBCHEM_ID=="NA") # 443
## remove "NA"
cmap2pub <- cmap2pub[cmap2pub$PUBCHEM_ID != "NA",] # 866 X 2
dup <- cmap2pub[cmap2pub$PUBCHEM_ID %in% cmap2pub$PUBCHEM_ID[duplicated(cmap2pub$PUBCHEM_ID)],]
cmapAnnot2[cmapAnnot2$cmap_id %in% dup$cmap_id,]
### two cmap drugs with different names have the same pubchem cid, there are three this situation
# cmap_id to chembl id mapping
cmap2pub$PUBCHEM_ID <- gsub("CID", "", cmap2pub$PUBCHEM_ID)
library(dplyr)
cmap2chem <- cmap2pub %>% left_join(chem2pub, by=c("PUBCHEM_ID"="pubchem_cid"))
cmap2chem <- unique(na.omit(cmap2chem[,c("cmap_id", "chembl_id")])) # 800 X 2

cc_path <- "inst/scripts/compoundCollection.db"
cc_conn <- dbConnect(SQLite(), cc_path)
dbListTables(cc_conn)
id_mapping <- dbGetQuery(cc_conn, "SELECT * FROM id_mapping")
id_mapping2 <- merge(id_mapping, cmap2chem, by="chembl_id", all.x=TRUE, all.y=TRUE) # 7605 X 4
dbWriteTable(cc_conn, "id_mapping", id_mapping2, overwrite=TRUE)
dbWriteTable(cc_conn, "cmapAnnot", cmapAnnot2)
dbListTables(cc_conn)
dbDisconnect(cc_conn)

###########
## LINCS ##
###########

# get LINCS annotation table
url <- "ftp://ftp.ncbi.nlm.nih.gov/geo/series/GSE92nnn/GSE92742/suppl/GSE92742_Broad_LINCS_pert_info.txt.gz"
download.file(url, "./inst/scripts/GSE92742_Broad_LINCS_pert_info.txt.gz")
pertIDs <- read.delim("inst/scripts/GSE92742_Broad_LINCS_pert_info.txt.gz")
pertIDs <- pertIDs[pertIDs$pert_type=="trt_cp", ] # 20,413 X 8
lincsAnnot <- pertIDs
colnames(lincsAnnot)[1] <- "lincs_id"

# get lincs internal id (e.g. BRD-A00100033) to chembl id mapping via inchi_key
## Download and extract the ChEMBL SQLite database from 
## ftp://ftp.ebi.ac.uk/pub/databases/chembl/ChEMBLdb/releases/chembl_25/chembl_25_sqlite.tar.gz
## with 3.3GB. Replace the following path to the path of your downloaded ChEMBL db 
chembldb <- "~/insync/project/ChEMBL_data/chembl_25/chembl_25_sqlite/chembl_25.db"
library(RSQLite)
conn <- dbConnect(SQLite(), chembldb)
cmp_str <- dbGetQuery(conn, 'SELECT chembl_id, molregno, standard_inchi_key
                             FROM compound_structures, chembl_id_lookup
                             WHERE compound_structures.molregno = chembl_id_lookup.entity_id') # 3,099,738 X 3
lincs_inchi <- pertIDs[,c("pert_id", "inchi_key")]
lincs_inchi <- lincs_inchi[lincs_inchi$inchi_key != "-666",] # 20350 X 2
length(unique(lincs_inchi$inchi_key)) # 20307
library(dplyr)
lincs_inchi2 <- lincs_inchi %>% left_join(cmp_str[,c("chembl_id", "standard_inchi_key")],
                                          by=c("inchi_key"="standard_inchi_key"))
lincs2chem <- unique(na.omit(lincs_inchi2[,c("pert_id", "chembl_id")])) # 10,930 X 2
colnames(lincs2chem) <- c("lincs_id", "chembl_id")
length(unique(lincs2chem$lincs_id)) # 5774 unique lincs internal id
length(unique(lincs2chem$chembl_id)) # 10,878 unique chembl id
write_tsv(lincs2chem, "inst/scripts/lincs2chem.tsv")

# write annotation table and id_mapping table to SQLite db
cc_path <- "inst/scripts/compoundCollection.db"
cc_conn <- dbConnect(SQLite(), cc_path)
dbListTables(cc_conn)
id_mapping <- dbGetQuery(cc_conn, "SELECT * FROM id_mapping")
id_mapping2 <- merge(id_mapping, lincs2chem, by="chembl_id", all.x=TRUE, all.y=TRUE) # 17,052 X 5
dbWriteTable(cc_conn, "id_mapping", id_mapping2, overwrite=TRUE)
dbWriteTable(cc_conn, "lincsAnnot", lincsAnnot)
dbListTables(cc_conn)
dbDisconnect(cc_conn)

# write LINCS sdf file
library(ChemmineR)
library(ChemmineOB) # make sure openbabel module is loaded!!
pertIDs <- pertIDs[pertIDs$canonical_smiles!=-666, ]
smiset <- as.character(pertIDs$canonical_smiles)
pertids <- as.character(pertIDs$pert_id)
names(smiset) <- pertids
lincs_sdfset <- smiles2sdf(smiset)
valid <- validSDF(lincs_sdfset); lincs_sdfset <- lincs_sdfset[valid]
## note, original smiset contained 20350 and final valid SDFset 20333
saveRDS(lincs_sdfset, "inst/scripts/lincs_sdfset.rds")
lincs_sdfset <- readRDS("inst/scripts/lincs_sdfset.rds")
write.SDF(lincs_sdfset, file="inst/scripts/lincs.sdf")

# Save the 'cmap.sdf', 'lincs.sdf', 'drugage.sdf', 'drugbank.sdf', 'compoundCollection.db'
# files to '../ccdata/customCMPdb' directory and renamed as
# 'cmap02.sdf', 'lincs_pilot1.sdf', 'drugage_build2.sdf', 'drugbank_5.1.5.sdf'
# and 'compoundCollection_0.1.db'
# Upload the above files to AnnotationHub

###################################################
## Update the SQLite database to include compound annotation tables of DrugAge build 4 and LINCS beta 2020
###################################################

#### Update lincsAnnot to include more annotations from lincs_pert_info 2017 ####
#################################################################################

conn <- loadAnnot()
dbListTables(conn)
idmap <- dbReadTable(conn, "id_mapping")
cmpdb_lincs <- dbReadTable(conn, "lincsAnnot")
data("lincs_pert_info")
cmpdb_lincs2 <- tibble(left_join(cmpdb_lincs, select(lincs_pert_info, pert_id, MOA:selectivity_comment),
                          by=c(lincs_id="pert_id")))
dbWriteTable(conn, "lincsAnnot", cmpdb_lincs2, overwrite=TRUE)

#### Add DrugAge build 4 annotation table ####
##############################################

library(dplyr); library(readr); library(stringr); library(magrittr)
## clean DrugAge build 4 table
download.file("https://genomics.senescence.info/drugs/dataset.zip", 
              "~/insync/project/DrugAge/dataset_build4.zip")
unzip("~/insync/project/DrugAge/dataset_build4.zip", 
      exdir="~/insync/project/DrugAge/dataset_build4")
drugage <- read_csv("~/insync/project/DrugAge/dataset_build4/drugage.csv")

drugage_clean <- drugage %>% 
    select(-11, -12) %>%
    group_by(compound_name) %>% 
    summarise_all(paste, collapse="; ")

drugage_scrape <- lapply(seq_along(drugage_clean$compound_name), function(n) {
    cat("doing", n, "\n")
    system(paste0(
        'curl "https://genomics.senescence.info/drugs/drug_details.php?compound_name=',
        str_replace_all(drugage_clean$compound_name[n], " ", "%20"),
        '"'
    ), intern=TRUE) %>% 
        {
            cid <- if(length(cid_line <- str_which(., "<dt>PubChem CID")) > 0) {
                .[cid_line + 1] %>% str_extract("[0-9]+")
            } else NA
            iupac <- if(length(iupac_line <- str_which(., "<dt>IUPAC Name")) > 0) {
                .[iupac_line + 1] %>% str_remove("^[ ]+<dd>") %>% str_remove("&nbsp;</dd>\\\r$") 
            } else NA
            list(
                cid = cid,
                iupac = iupac
            )
        }
})

drugage_clean$pubchem_cid <- sapply(drugage_scrape, `[[`, "cid")
drugage_clean$iupac <- sapply(drugage_scrape, `[[`, "iupac")
drugage_clean$pubchem_cid[drugage_clean$compound_name == "1,2,4-triazolo[1,5-a]pyridine"] <- "67508"
drugage_clean$pubchem_cid[drugage_clean$compound_name == "7-cyclopentyl-5-(4-phenoxy)phenyl-7H-pyrrolo[2,3-d]pyrimidin-4-ylamine"] <- "6603792"
drugage_clean$pubchem_cid[drugage_clean$compound_name == "N-(2-[4-(4-chlorophenyl)piperazin-1-yl]ethyl)-3-methoxybenzamide"] <- "3626837"

## get pubchem cid to ChEMBL id mapping table
download.file("ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src22.txt.gz",
              "~/insync/project/UniChem/chembl2pubchemcid.txt.gz") # download at 2022/01/31
chem2pub <- read_tsv("~/insync/project/UniChem/chembl2pubchemcid.txt.gz") # 2027837 X 2
colnames(chem2pub) <- c("chembl_id", "pubchem_cid")
chem2pub$pubchem_cid <- as.character(chem2pub$pubchem_cid)
drugage_clean %<>% left_join(chem2pub, by = "pubchem_cid")
drugage_clean %>% print(width=Inf)
write_tsv(drugage_clean, "~/insync/project/DrugAge/data/drugage4_annot.tsv")
## Add to SQLite
addCustomAnnot(drugage_clean, id_col="compound_name", annot_name="drugAge4")

da4 <- dbReadTable(conn, "drugAge4")
idmap <- getidmap()

#### Add curated LINCS beta 2020 compound annotation table ####
###############################################################
data("lincs_pert_info2")
addCustomAnnot(lincs_pert_info2, id_col="pert_id", annot_name="lincs2")

cmplincs2 <- dbReadTable(conn, "lincs2")
idmap <- getidmap()
sample_n(idmap, 50)
## copy and rename the cached SQLite
cache_path <- ah[["AH79563"]]
file.copy(cache_path, "~/insync/project/ccdata/customCMPdb/compoundCollection_0.1.db", 
          overwrite=TRUE)
dbDisconnect(conn)
## Double check
conn2 <- dbConnect(SQLite(), "~/insync/project/ccdata/customCMPdb/compoundCollection_0.1.db")
dbListTables(conn2)
idmap <- dbReadTable(conn2, "id_mapping")
sample_n(idmap, 50)
lincs_update <- dbReadTable(conn2, "lincsAnnot")
lincs2 <- dbReadTable(conn2, "lincs2")
da4 <- dbReadTable(conn2, "drugAge4")

## Update inst/extdata/metadata.csv, ask for Bioconductor core team to update the compoundCollection_0.1.db file.
## And upload the above files to AnnotationHub after their reply.

## ----setup, echo=FALSE, messages=FALSE, warnings=FALSE------------------------
suppressPackageStartupMessages({
  library(customCMPdb); library(ChemmineR)
})

## ----install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("customCMPdb")

## ----inst_git, eval=FALSE-----------------------------------------------------
# devtools::install_github("yduan004/customCMPdb", build_vignettes=TRUE)

## ----load, eval=TRUE, message=FALSE-------------------------------------------
library(customCMPdb)
library(help = "customCMPdb")  # Lists package info

## ----load_vignette, eval=FALSE, message=FALSE---------------------------------
# browseVignettes("customCMPdb")  # Opens vignette

## ----sql, eval=TRUE, message=FALSE--------------------------------------------
conn <- loadAnnot()
library(RSQLite)
dbListTables(conn)
drugAgeAnnot <- dbReadTable(conn, "drugAgeAnnot")
head(drugAgeAnnot)
dbDisconnect(conn)

## ----da, eval=TRUE, message=FALSE, results=FALSE------------------------------
da_sdfset <- loadSDFwithName(source="DrugAge")

## ----da_chemmineR, eval=TRUE, message=FALSE, results=FALSE--------------------
ChemmineR::cid(da_sdfset) <- ChemmineR::sdfid(da_sdfset)
ChemmineR::plot(da_sdfset[1])

## ----db, eval=FALSE-----------------------------------------------------------
# db_sdfset <- loadSDFwithName(source="DrugBank")

## ----cmap, eval=TRUE, message=FALSE, results=FALSE----------------------------
cmap_sdfset <- loadSDFwithName(source="CMAP2")

## ----lincs, eval=TRUE, message=FALSE, results=FALSE---------------------------
lincs_sdfset <- loadSDFwithName(source="LINCS")

## ----download_db, eval=TRUE, message=FALSE------------------------------------
library(AnnotationHub)
ah <- AnnotationHub()
annot_path <- ah[["AH79563"]]

## ----custom, eval=TRUE, message=FALSE, results=FALSE--------------------------
chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
               "CHEMBL100", "CHEMBL1000", NA)
annot_tb <- data.frame(cmp_name=paste0("name", 1:6),
        chembl_id=chembl_id,
        feature1=paste0("f", 1:6),
        feature2=rnorm(6))
addCustomAnnot(annot_tb, annot_name="myCustom")

## ----del, eval=TRUE, message=FALSE--------------------------------------------
listAnnot()
deleteAnnot("myCustom")
listAnnot()

## ----default, eval=FALSE------------------------------------------------------
# defaultAnnot()

## ----query, eval=TRUE, message=FALSE------------------------------------------
query_id <- c("CHEMBL1064", "CHEMBL10", "CHEMBL113", "CHEMBL1004", "CHEMBL31574")
listAnnot()
qres <- queryAnnotDB(query_id, annot=c("drugAgeAnnot", "lincsAnnot"))
qres
# query the added custom annotation
addCustomAnnot(annot_tb, annot_name="myCustom")
qres2 <- queryAnnotDB(query_id, annot=c("lincsAnnot", "myCustom"))
qres2

## ----not_chembl, eval=TRUE, message=FALSE-------------------------------------
query_id <- c("BRD-A00474148", "BRD-A00150179", "BRD-A00763758", "BRD-A00267231")
qres3 <- queryAnnotDB(chembl_id=query_id, annot=c("lincsAnnot"))
qres3

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

library(AnnotationHub)
library(AnnotationHubData)
meta <- data.frame(
  Title = c("annot_0.1", 
            "drugage_build2", "drugbank_5.1.5", "cmap02", "lincs_pilot1"),
  Description = c("SQLite database containing compound annotations from four sources",
                  "SDF file of compounds in DrugAge database",
                  "SDF file of compounds in DrugBank database",
                  "SDF file of compounds in CMAP02 database",
                  "SDF file of compounds in LINCS database"),
  BiocVersion = rep("3.11", 5),
  Genome = rep("GRCh38", 5),
  # getValidSourceTypes()
  SourceType = c("TSV", "XLS/XLSX", "TXT",
                 "TSV", "TSV"),
  SourceUrl = c("https://bioconductor.org/packages/release/bioc/html/customCMPdb.html",
                "https://genomics.senescence.info/drugs/dataset.zip",
                "https://www.drugbank.ca/releases/latest#structures",
                "https://portals.broadinstitute.org/cmap/",
                "ftp://ftp.ncbi.nlm.nih.gov/geo/series/GSE92nnn/GSE92742/suppl/GSE92742_Broad_LINCS_pert_info.txt.gz"),
  SourceVersion = c("0.1", 
                    "Build 2", "5.1.5",
                    "Build02", "Pilot Phase 1"),
  Species = rep("Homo sapiens", 5),
  TaxonomyId = rep(9606, 5),
  Coordinate_1_based = rep(TRUE, 5),
  DataProvider = c("DrugAge, DrugBank, Broad Institute",
                   "DrugAge", "DrugBank",
                   "Broad Institute", "Broad Institute"),
  Maintainer = rep("Yuzhu Duan <yduan004@ucr.edu>", 5),
  RDataClass = c("character", 
                 "character", "character",
                 "character", "character"),
  # AnnotationHub::DispatchClassList()
  DispatchClass = c("FilePath", 
                    "FilePath", "FilePath",
                    "FilePath", "FilePath"),
  RDataPath = c("customCMPdb/compoundCollection_0.1.db",
                "customCMPdb/drugage_build2.sdf",
                "customCMPdb/drugbank_5.1.5.sdf",
                "customCMPdb/cmap02.sdf",
                "customCMPdb/lincs_pilot1.sdf"),
  Tags = c("annot", "drugage", "drugbank",
           "cmap", "lincs")
)

write.csv(meta, file="inst/extdata/metadata.csv", row.names=FALSE)

AnnotationHubData::makeAnnotationHubMetadata("../customCMPdb")

#' Annotation Queries with Compound IDs
#'
#' This function can be used to query compound annotations from the default
#' resources as well as the custom resources stored in the SQLite annotation
#' database. The default annotation resources are DrugAge, DrugBank, CMAP02 and
#' LINCS. The customized compound annotations could be added/deleted by the
#' \code{\link{customAnnot}} utilities.
#' 
#' @details 
#' The input of this query function could be a set of ChEMBL IDs, it returns a
#' data.frame storing annotations of the input compounds from selected
#' annotation resources defined by the \code{annot} argument.
#' 
#' Since in the SQLite annotation database, ID identifiers from different ID systems, 
#' such as DrugBank and LINCS, are connected by ChEMBL IDs, it is hard to tell 
#' whether two IDs, such as DB00341, BRD-A42571354, refer to the same compound if 
#' either of them lack ID mappings to ChEMBL. So for querying compounds that don't 
#' have ChEMBL IDs, only one isolated database where the compounds belong to are 
#' supported. For example, a compound with LINCS id as "BRD-A00150179" doesn't have 
#' the ChEMBL ID mapping, when it is passed to the `chembl_id` argument, 
#' the `annot` need only to be set as `lincsAnnot` and the result
#' will be the compound annotation table from the LINCS annotation.
#'
#' @param chembl_id character vector of ChEMBL IDs or compound ids from other 
#' annotation system..
#' @param annot character vector of annotation resources, such as
#' \code{drugAgeAnnot}, \code{DrugBankAnnot}, \code{cmapAnnot}, \code{lincsAnnot} 
#' or names of the annotation tables added by users. The \code{\link{listAnnot}}
#' function lists the available options for the \code{annot} argument names.
#' @return data.frame of annotation result
#' @import RSQLite
#' @examples
#' query_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL100109",
#'                "CHEMBL100", "CHEMBL1000", "CHEMBL10")
#' qres <- queryAnnotDB(query_id, annot=c("drugAgeAnnot", "lincsAnnot"))
#'
#' # Add a custom compound annotation table
#' chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
#'                "CHEMBL100", "CHEMBL1000", NA)
#' annot_tb <- data.frame(compound_name=paste0("name", 1:6),
#'         chembl_id=chembl_id,
#'         feature1=paste0("f", 1:6),
#'         feature2=rnorm(6))
#' addCustomAnnot(annot_tb, annot_name="myCustom2")
#'
#' # query custom annotation
#' qres2 <- queryAnnotDB(query_id, annot=c("lincsAnnot", "myCustom2"))
#' 
#' # query compounds that don't have ChEMBL IDs
#' query_id <- c("BRD-A00474148", "BRD-A00150179", "BRD-A00763758", "BRD-A00267231")
#' qres3 <- queryAnnotDB(chembl_id=query_id, annot=c("lincsAnnot"))
#' qres3
#' @export
queryAnnotDB <- function(chembl_id,
        annot=c("drugAgeAnnot", "DrugBankAnnot", "cmapAnnot", "lincsAnnot")){
    ah <- AnnotationHub()
    dbpath <- ah[["AH79563"]]
    conn <- dbConnect(SQLite(), dbpath)
    
    da_cols <- c("species", "strain", "dosage", "avg_lifespan_change",
                 "max_lifespan_change", "gender", "significance")
    db_cols <- c("drugbank_id", "name", "cas.number", "unii", "state",
                 "groups", "fda.label")
    cmap_cols <- c("instance_id", "concentration..M.", "duration..h.", "cell2",
                   "array3", "perturbation_scan_id", "vehicle_scan_id4",
                   "scanner", "catalog_number")
    lincs_cols <- c("lincs_id", "pert_iname", "is_touchstone", "inchi_key",
                    "pubchem_cid")
    
    if(isCHEMBL(chembl_id)){
        chembl_id <- paste0("(\"", paste(chembl_id, collapse="\", \""), "\")")
        query <- dbSendQuery(conn, paste("SELECT a.chembl_id, a.drugbank_id,
            a.lincs_id, b.species, b.strain, b.dosage, b.avg_lifespan_change,
            b.max_lifespan_change, b.gender, b.significance,
            c.name, c.'cas-number', c.unii, c.state, c.groups, c.'fda-label',
            d.instance_id, d.'concentration..M.', d.'duration..h.',
            d.cell2, d.array3, d.perturbation_scan_id, d.vehicle_scan_id4,
            d.scanner, d.catalog_number,
            e.pert_iname, e.is_touchstone, e.inchi_key, e.pubchem_cid
            FROM id_mapping AS a
            LEFT JOIN drugAgeAnnot AS b ON a.drugage_id = b.drugage_id
            LEFT JOIN DrugBankAnnot AS c ON a.drugbank_id = c.drugbank_id
            LEFT JOIN cmapAnnot AS d ON a.cmap_id = d.cmap_id
            LEFT JOIN lincsAnnot AS e ON a.lincs_id = e.lincs_id
            WHERE a.chembl_id IN", chembl_id,
            "GROUP BY a.chembl_id
            ORDER BY a.chembl_id"))
        assays <- dbFetch(query)
        dbClearResult(query)
        result <- data.frame(assays)
        
        res <- result[, "chembl_id", drop=FALSE]
        for(x in annot){
            if(x == "drugAgeAnnot") res <- cbind(res, result[,da_cols])
            if(x == "DrugBankAnnot") res <- cbind(res, result[,db_cols])
            if(x == "cmapAnnot") res <- cbind(res, result[,cmap_cols])
            if(x == "lincsAnnot") res <- cbind(res, result[,lincs_cols])
            if(! x %in% c("drugAgeAnnot", "DrugBankAnnot", "cmapAnnot", "lincsAnnot")){
                idcol <- paste0(x, "_id")
                cust_annot <- dbGetQuery(conn, paste0("SELECT a.chembl_id, a.",
                                  idcol, ", b.* FROM id_mapping AS a LEFT JOIN ", x,
                                  " AS b ON a.", idcol, " = b.", idcol,
                                  " WHERE a.chembl_id IN", chembl_id,
                                  " GROUP BY a.chembl_id
                                  ORDER BY a.chembl_id"))
                cust_annot <- cust_annot[, !colnames(cust_annot) %in%
                                             c("chembl_id", idcol)]
                res <- cbind(res, cust_annot)
            }
        }
    } else {
        if(length(annot) > 1) 
            stop("The query ids passed to 'chembl_id' argument are not identified as ChEMBL IDs,
                 so only one annotation resource where the query id come frome is supported.")
        annot_tb <- dbReadTable(conn, annot)
        index <- apply(annot_tb, 1, function(r) any(chembl_id %in% r))
        res <- annot_tb[index,]
        if(annot == "drugAgeAnnot") res <- res[,da_cols]
        if(annot == "DrugBankAnnot") res <- res[,db_cols]
        if(annot == "cmapAnnot") res <- res[,cmap_cols]
        if(annot == "lincsAnnot") res <- res[,lincs_cols]
    }
    dbDisconnect(conn)
    return(res)
}

########################################
## Import of DrugBank Annotation Data ##
########################################
## Function to import DrugBank xml to data.frame and store in SQLite database.
## Note, this functions needs some major speed improvements. Ideally,
## (1) Download
##     - download DrugBank xml file (https://www.drugbank.ca/releases/latest)
##     - name uncompressed file 'drugbank.xml'
## (2) Function to convert xml into dataframe and store in SQLite database.

#' @export
#' @importFrom XML xmlParse
#' @importFrom XML xmlRoot 
#' @importFrom XML xmlSize 
#' @importFrom XML xmlToDataFrame 
#' 
#' @title Convert drugbank database (xml file) into dataframe.
#' 
#' @description Download the original DrugBank database 
#' at \url{http://www.drugbank.ca/releases/latest} (xml file) into your current 
#' working directory and rename as "drugbank.xml"
#' then run: 
#' \code{drugbank_df = dbxml2df(xmlfile="drugbank.xml", version="5.0.10")}.
#' 
#' @param xmlfile Character(1), file path to the xml file downloaded from the
#' DrugBank website at \url{https://www.drugbank.ca/releases/latest}
#' @param version Character(1), DrugBank version of the xml file
#' @return Dataframe of drugbank xml database.
#' @references \url{http://www.drugbank.ca/releases/latest}
#' @author Yuzhu Duan \url{yduan004@ucr.edu}
#' @note This process with take about 20 minutes.
#' @seealso \code{\link{df2SQLite}}
#' @aliases dbxml2df
#' @examples
#' library(XML)
#' \dontrun{
#' ## download the original drugbank database at 
#' \url{http://www.drugbank.ca/releases/latest} (xml file)
#' ## into your current directory and rename as drugbank.xml
#' 
#' ## convert drugbank database (xml file) into dataframe: 
#' drugbank_df <- dbxml2df(xmlfile="drugbank.xml", version="5.0.10")
#' }
dbxml2df <- function(xmlfile, version) {
    myxml <- xmlParse(file=xmlfile) 
    rootnode <- xmlRoot(myxml)
    rootsize <- xmlSize(rootnode)
    mycol <- c("drugbank-id", "name", "description", "cas-number", "unii", 
           "state", "groups", "general-references", "synthesis-reference",
           "indication", "pharmacodynamics", "mechanism-of-action", "toxicity", 
           "metabolism", "absorption", "half-life", "protein-binding",
           "route-of-elimination", "volume-of-distribution", "clearance",
           "classification", "salts", "synonyms", "products", "international-brands",
           "mixtures", "packagers", "manufacturers", "prices", "categories",
           "affected-organisms", "dosages", "atc-codes", "ahfs-codes", "pdb-entries",
           "fda-label", "msds", "patents", "food-interactions", "drug-interactions",
           "sequences", "experimental-properties", "external-identifiers", "external-links",
           "pathways", "reactions", "snp-effects", "snp-adverse-drug-reactions", "targets",
           "enzymes", "carriers", "transporters", "average-mass", "monoisotopic-mass", 
           "calculated-properties")
    ## (b) Extract corresponding data in loop and inject into preformatted data.frame 
    message("Extracting data for column names. This may take 20 minutes.")
    df <- as.data.frame(matrix(NA, nrow=rootsize, ncol=length(mycol), dimnames=list(1:rootsize, mycol)))
    for(i in 1:rootsize) {
        tmp <- xmlToDataFrame(rootnode[i], stringsAsFactors = FALSE, collectNames = FALSE)
        v <- as.character(tmp[1,]); names(v) <- colnames(tmp)
        df[i,] <- v[mycol]
    }
    message("Successfully convert DrugBank database (xml file) into dataframe.")
    return(df)
}

#' @importFrom utils read.csv
#' @importFrom utils unzip
#' 
#' @title Store drugbank dataframe into an SQLite database 
#' @description Store specific version of drugbank dataframe into an SQLite database 
#' under user defined directory, the default is user's present working directory 
#' of R session
#' @param dbdf Drugbank dataframe generated by \code{\link{dbxml2df}} function.
#' @param version Character(1), version of the input drugbank dataframe generated 
#' by \code{\link{dbxml2df}} function
#' @param dest_dir Character(1), destination directory that the result SQLite 
#' database stored in. The default is user's current working directory
#' @return SQLite database named as "drugbank_<versionNumber>.db" stored under user's 
#' present working directory of R session or user's specified directory.
#' @author Yuzhu Duan \url{yduan004@ucr.edu}
#' @seealso \code{\link{dbxml2df}} 
#' @aliases df2SQLite
#' @examples 
#' library(RSQLite)
#' \dontrun{
#' # download the original drugbank database (http://www.drugbank.ca/releases/latest) (xml file) 
#' # to your current R working directory, and rename as "drugbank.xml".
#' # Read in the xml file and convert to a data.frame in R 
#' 
#' drugbank_df = dbxml2df(xmlfile="drugbank.xml", version="5.1.5")
#' 
#' # store the converted drugbank dataframe into SQLite database under user's
#' present R working direcotry, or other directory defined by 'dest_dir'
#'
#' df2SQLite(dbdf=drugbank_df, version="5.1.5") # set version as version of xml file
#' }
#' @export
df2SQLite <- function(dbdf, version, dest_dir="."){
  mydb <- dbConnect(SQLite(), paste0(dest_dir, "/drugbank_",version,".db"))
  RSQLite::dbWriteTable(mydb, "dbdf", dbdf)
  dbDisconnect(mydb)
    message("Successfully store drugbank dataframe into SQLite database named as", 
            paste0("'drugbank_",version,".db'"), 
            " and it is under 'dest_dir'")
}

#‘ 
#' Load Compound Structures from Four Resources
#'
#' This function could be used to get SDFset of compounds in CMAP2, LINCS 2017, 
#' DrugAge build 2 or DrugBank 5.1.5 databases. The \code{cid} of the SDFset are 
#' compound names instead of their internal IDs.
#' 
#' @param source character(1), one of "CMAP2", "LINCS", "DrugBank", "DrugAge"
#' @return SDFset object of compounds in the \code{source} database, the \code{cid}
#' of the SDFset are compound names.
#' @seealso 
#' \code{\link[ChemmineR]{SDFset}}
#' @examples
#' da_sdf <- loadSDFwithName(source="DrugAge")
#' @export
loadSDFwithName <- function(source="LINCS"){
    ah <- AnnotationHub()
    annot_path <- ah[["AH79563"]]
    conn <- dbConnect(SQLite(), annot_path)
    if(source == "LINCS"){
        lincs_annot <- dbReadTable(conn, "lincsAnnot")
        lincs_sdf_path <- ah[["AH79567"]]
        lincs_sdf <-ChemmineR::read.SDFset(lincs_sdf_path)
        cid(lincs_sdf) <- sdfid(lincs_sdf)
        ## make lincs cids as pert names
        brd_ids <- lincs_annot$lincs_id
        names(brd_ids) <- lincs_annot$pert_iname
        brd_uniq <- brd_ids[!duplicated(names(brd_ids))] # 19811
        brd_common <- brd_uniq[brd_uniq %in% cid(lincs_sdf)] # 19758
        res_sdf <- lincs_sdf[brd_common]
        cid(res_sdf) <- names(brd_common)
    }
    if(source == "CMAP2"){
        cmap_annot <- dbReadTable(conn, "cmapAnnot")
        cmap_sdf_path <- ah[["AH79566"]]
        cmap_sdf <-ChemmineR::read.SDFset(cmap_sdf_path)
        cid(cmap_sdf) <- sdfid(cmap_sdf)
        cmap_ids <- cmap_annot$cmap_id
        names(cmap_ids) <- cmap_annot$cmap_name
        res_sdf <- cmap_sdf[cmap_ids]
        cid(res_sdf) <- names(cmap_ids)
    }
    if(source == "DrugBank"){
        db_annot <- dbReadTable(conn, "drugBankAnnot")
        db_sdf_path <- ah[["AH79565"]]
        db_sdf <-ChemmineR::read.SDFset(db_sdf_path)
        cid(db_sdf) <- sdfid(db_sdf)
        db_ids <- db_annot$drugbank_id
        names(db_ids) <- db_annot$name
        db_common <- db_ids[db_ids %in% cid(db_sdf)] # 10569
        res_sdf <- db_sdf[db_common]
        cid(res_sdf) <- names(db_common)
    }
    if(source == "DrugAge"){
        da_annot <- dbReadTable(conn, "drugAgeAnnot")
        da_sdf_path <- ah[["AH79564"]]
        da_sdf <-ChemmineR::read.SDFset(da_sdf_path)
        cid(da_sdf) <- sdfid(da_sdf)
        da_ids <- da_annot$drugage_id
        names(da_ids) <- da_annot$compound_name
        da_uniq <- da_ids[!duplicated(names(da_ids))] # 420
        da_common <- da_uniq[da_uniq %in% cid(da_sdf)] # 223
        res_sdf <- da_sdf[da_common]
        cid(res_sdf) <- names(da_common)
    }
    dbDisconnect(conn)
    return(res_sdf)
}

isCHEMBL <- function(id){
    return(all(grepl("CHEMBL", na.omit(id))))
}

#' Load Compound Annotation Database
#' 
#' The compound annotation tables from different databases/sources are stored
#' in one SQLite database. This function can be used to load the SQLite 
#' annotation database
#' @examples 
#' conn <- loadAnnot()
#' @export
loadAnnot <- function(){
    ah <- AnnotationHub()
    # query(ah, c("customCMPdb", "annot_0.1"))
    annot_path <- ah[["AH79563"]]
    conn <- dbConnect(SQLite(), annot_path)
    return(conn)
}

#' Add/Delete Custom Annotation
#'
#' Functions could be used to add/delete user's custom compound annotations
#' from the annotation SQLite database.
#' The added custom compound annotation table should contains a column named as
#' \code{chembl_id} that represents the ChEMBL ids of the added compounds.
#'
#' @rdname customAnnot
#' @aliases addCustomAnnot customAnnot deleteAnnot listAnnot defaultAnnot
#' @param annot_tb data.frame representing the custom annotation table,
#' Note, it should contains a 'chembl_id' column representing the compound
#' ChEMBL ids
#' @param id_col column name in \code{annot_tb} that is used as ID column, 
#' this column must contain unique identifiers. If not defined, an automatically
#' generated ID column will be appended.
#' @param annot_name character(1), user defined name of the annotation table
#' @param overwrite a logical specifying whether to overwrite an existing table 
#' or not. Its default is FALSE.
#' @importFrom stats na.omit
#' @examples
#' chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
#'                "CHEMBL100", "CHEMBL1000", NA)
#' annot_tb <- data.frame(compound_name=paste0("name", 1:6),
#'         chembl_id=chembl_id,
#'         feature1=paste0("f", 1:6),
#'         feature2=rnorm(6))
#' addCustomAnnot(annot_tb, annot_name="mycustom3")
#' @export
addCustomAnnot <- function(annot_tb, id_col=NULL, annot_name, overwrite=FALSE){
    # check validity of annot_tb
    if(! "chembl_id" %in% colnames(annot_tb)){
        stop("The input annot_tb does not contain a 'chembl_id' column,
             Please make sure that your custom comppounds have ChEMBL ids!")
    }
    if(! isCHEMBL(annot_tb$chembl_id)){
        stop("The ChEMBL ids are not in the correct format, please check!")
    }
    # check whether annot_name already exists in SQLite
    conn <- loadAnnot()
    annot_names <- dbListTables(conn)
    if(tolower(annot_name) %in% tolower(annot_names) & !overwrite){
        # ans <- readline(paste("This name exists in the SQLite database,",
        #                     "do you want to overwrite it? (yes/no)"))
        # if(tolower(ans)=="no" | ans==""){
        #     dbDisconnect(conn)
        #     return()
        # }
        warning("The annot_name exists in the SQLite database, the old table is not overwritten, set 'overwrite=TRUE' to overwrite the existing one")
        dbDisconnect(conn)
        return()
    }
    ndigit <- nchar(as.character(nrow(annot_tb)))
    iid_name <- paste0(annot_name, "_id")
    if(is.null(id_col)){
        annot_tb <- data.frame(
            internal_id=paste0(toupper(annot_name),
                               sprintf(paste0("%0", ndigit, "d"), seq_len(nrow(annot_tb)))),
            annot_tb
        )
        colnames(annot_tb)[1] <- iid_name
    } else {
        if(sum(duplicated(annot_tb[[id_col]])) > 0 | sum(is.na(annot_tb[[id_col]])) > 0){
            stop("The id_col of annot_tb need to be unique and doesn't contain NA values!")
        }
        annot_tb <- data.frame(
            internal_id=annot_tb[[id_col]],
            annot_tb
        )
        colnames(annot_tb)[1] <- iid_name
    }
    
    chem2in <- na.omit(annot_tb[ ,c("chembl_id", iid_name)])

    # write annotation table and id_mapping table to SQLite db
    id_mapping <- dbReadTable(conn, "id_mapping")
    id_mapping <- id_mapping[,colnames(id_mapping) != iid_name]
    id_mapping2 <- merge(id_mapping, chem2in, by="chembl_id",
                         all.x=TRUE, all.y=TRUE)
    dbWriteTable(conn, "id_mapping", id_mapping2, overwrite=TRUE)

    dbWriteTable(conn, annot_name,
                 annot_tb,
                 overwrite=TRUE)
    message("The SQLite database now contains the following tables:\n",
            paste(dbListTables(conn), collapse=" "))
    dbDisconnect(conn)
}

#' @rdname customAnnot
#' @examples
#' deleteAnnot("mycustom3")
#' @export
deleteAnnot <- function(annot_name){
    if(tolower(annot_name) %in% c("cmapannot", "drugageannot",
                                  "drugbankannot", "lincsannot", "drugage4", "lincs2")){
        stop("The default annotation resources could not be deleted!")
    }
    conn <- loadAnnot()
    annot_names <- dbListTables(conn)
    if(! annot_name %in% annot_names){
        dbDisconnect(conn)
        stop("The 'annot_name' does not exist in the SQLite database!")
    }
    id_map <- dbReadTable(conn, "id_mapping")
    # delete annot_name column in id_mapping table
    id_map <- id_map[,colnames(id_map) != paste(annot_name, "id", sep="_")]
    # remove rows that are all NAs except for chembl_id column
    tmp <- id_map[,colnames(id_map) != "chembl_id"]
    del_rows <- which(apply(is.na(tmp), 1, all))
    if(length(del_rows) > 0){
        id_map <- id_map[-del_rows, ]
    }
    dbWriteTable(conn, "id_mapping", id_map, overwrite=TRUE)
    # delete annot_name table
    dbRemoveTable(conn, annot_name)
    message("The SQLite database now contains the following tables:\n",
            paste(dbListTables(conn), collapse=" "))
    dbDisconnect(conn)
}

#' @description The \code{listAnnot} function lists the available annotation
#' resources in the SQLite annotation database.
#' @importFrom AnnotationHub AnnotationHub
#' @return character vector of names of the annotation tables in the SQLite DB
#' @rdname customAnnot
#' @examples
#' annot_names <- listAnnot()
#' @export
listAnnot <- function(){
    conn <- loadAnnot()
    tb_names <- dbListTables(conn)
    annot_names <- tb_names[tb_names != "id_mapping"]
    dbDisconnect(conn)
    print(annot_names)
}

#' @rdname customAnnot
#' @description The \code{defaultAnnot} function sets the annotation SQLite
#' database to the default one by deleting the existing one and re-downloading
#' from AnnotationHub.
#' @return character(1), path to the annotation SQLite database
#' @examples
#' # defaultAnnot()
#' @export
defaultAnnot <- function(){
    ah <- AnnotationHub()
    annot_path <- ah[["AH79563"]]
    unlink(annot_path)
    annot_path <- ah[["AH79563"]]
    return(annot_path)
}

getidmap <- function(){
    conn <- loadAnnot()
    id_map <- dbReadTable(conn, "id_mapping")
    dbDisconnect(conn)
    return(id_map)
}

---
title: "_customCMPdb_: Integrating Community and Custom Compound Collections"
author: "Authors: Yuzhu Duan, Dan Evans, Kevin Horan, Austin Leong, Siddharth Sai and Thomas Girke"
date: "Last update: `r format(Sys.time(), '%d %B, %Y')`" 
output:
  BiocStyle::html_document:
    toc_float: true
    code_folding: show
vignette: >
  %\VignetteIndexEntry{customCMPdb}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
fontsize: 14pt
bibliography: bibtex.bib
editor_options: 
  chunk_output_type: console
---

<style>
pre code {
  white-space: pre !important;
  overflow-x: scroll !important;
  word-break: keep-all !important;
  word-wrap: initial !important;
}
</style>

<!---
- Compile from command-line
Rscript -e "rmarkdown::render('customCMPdb.Rmd', c('BiocStyle::html_document', 'pdf_document')); 
knitr::knit('customCMPdb.Rmd', tangle=TRUE)"
-->

```{r setup, echo=FALSE, messages=FALSE, warnings=FALSE}
suppressPackageStartupMessages({
  library(customCMPdb); library(ChemmineR)
})
```

# Introduction

This package serves as a query interface for important community collections of
small molecules, while also allowing users to include custom compound
collections. Both annotation and structure information is provided.  The
annotation data is stored in an SQLite database, while the structure
information is stored in Structure Definition Files (SDF). Both are hosted 
on Bioconductor's `AnnotationHub`. A detailed description of the included 
data types is provided under the _Supplemental Material_ section of this vignette. 
At the time of writing, the following community databases are included: 

+ [DrugAge](https://genomics.senescence.info/drugs/) [@Barardo2017-xk]
+ [DrugBank](https://www.drugbank.ca/) [@Wishart2018-ap]
+ [CMAP02](https://portals.broadinstitute.org/cmap/) [@Lamb2006-du]
+ [LINCS](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742) [@Subramanian2017-fu]

In addition to providing access to the above compound collections, the package
supports the integration of custom collections of compounds, that will be
automatically stored for the user in the same data structure as the
preconfigured databases. Both custom collections and those provided by this
package can be queried in a uniform manner, and then further analyzed with
cheminformatics packages such as `ChemmineR`, where SDFs are imported into
flexible S4 containers [@Cao2008-np].

# Installation and Loading
As Bioconductor package `customCMPdb` can be installed with the 
`BiocManager::install()` function.
```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("customCMPdb")
```

To obtain the most recent updates of the package immediately, one can also install it 
directly from GitHub as follows.
```{r inst_git, eval=FALSE}
devtools::install_github("yduan004/customCMPdb", build_vignettes=TRUE)
```

Next the package needs to be loaded in a user's R session.
```{r load, eval=TRUE, message=FALSE}
library(customCMPdb)
library(help = "customCMPdb")  # Lists package info
```

Open vignette of this package.
```{r load_vignette, eval=FALSE, message=FALSE}
browseVignettes("customCMPdb")  # Opens vignette
```

# Overview
The following introduces how to load and query the different datasets.

## DrugAge Annotations
The compound annotation tables are stored in an SQLite database. This data can be 
loaded into a user's R session as follows (here for `drugAgeAnnot`).

```{r sql, eval=TRUE, message=FALSE}
conn <- loadAnnot()
library(RSQLite)
dbListTables(conn)
drugAgeAnnot <- dbReadTable(conn, "drugAgeAnnot")
head(drugAgeAnnot)
dbDisconnect(conn)
```

## DrugAge SDF
The corresponding structures for the above DrugAge example can be loaded into an `SDFset` 
object as follows.
```{r da, eval=TRUE, message=FALSE, results=FALSE}
da_sdfset <- loadSDFwithName(source="DrugAge")
```

Instructions on how to work with `SDFset` objects are provided in the `ChemmineR` vignette 
[here](https://bioconductor.org/packages/ChemmineR/). For instance, one can plot any of 
the loaded structures with the `plot` function.

```{r da_chemmineR, eval=TRUE, message=FALSE, results=FALSE}
ChemmineR::cid(da_sdfset) <- ChemmineR::sdfid(da_sdfset)
ChemmineR::plot(da_sdfset[1])
```

## DrugBank SDF
The SDF from DrugBank can be loaded into R the same way.  The
corresponding SDF file was downloaded from
[here](https://www.drugbank.ca/releases/latest#structures). During the import
into R `ChemmineR` checks the validity of the imported compounds.

```{r db, eval=FALSE}
db_sdfset <- loadSDFwithName(source="DrugBank")
```

## CMAP SDF

The import of the SDF of the CMAP02 database works the same way.
```{r cmap, eval=TRUE, message=FALSE, results=FALSE}
cmap_sdfset <- loadSDFwithName(source="CMAP2")
```

## LINCS SDF

The same applies to the SDF of the small molecules included in the LINCS database.
```{r lincs, eval=TRUE, message=FALSE, results=FALSE}
lincs_sdfset <- loadSDFwithName(source="LINCS")
```

For reproducibility, the R code for generating the above datasets is included
in the `inst/scripts/make-data.R` file of this package. The file location 
on a user's system can be obtained with `system.file("scripts/make-data.R", 
package="customCMPdb")`.

# Custom Annotation Database 
## Load Annotation Database
The SQLite Annotation Database is hosted on Bioconductor's `AnnotationHub`. 
Users can download it to a local `AnnotationHub` cache directory. The path to this
directory can be obtained as follows.
```{r download_db, eval=TRUE, message=FALSE}
library(AnnotationHub)
ah <- AnnotationHub()
annot_path <- ah[["AH79563"]]
```

## Add Custom Annotation Tables
The following introduces how users can import to the SQLite database 
their own compound annotation tables. In this case, the corresponding
ChEMBL IDs need to be included under the `chembl_id` column. 
The name of the custom data set can be specified under the `annot_name` 
argument. Note, this name is case insensitive. 
```{r custom, eval=TRUE, message=FALSE, results=FALSE}
chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
               "CHEMBL100", "CHEMBL1000", NA)
annot_tb <- data.frame(cmp_name=paste0("name", 1:6),
        chembl_id=chembl_id,
        feature1=paste0("f", 1:6),
        feature2=rnorm(6))
addCustomAnnot(annot_tb, annot_name="myCustom")
```

## Delete Custom Annotation Tables
The following shows how to delete custom annotation tables
by referencing them by their name. To obtain a list of custom
annotation tables present in the database, the `listAnnot` function 
can be used.
```{r del, eval=TRUE, message=FALSE}
listAnnot()
deleteAnnot("myCustom")
listAnnot()
```

## Set to Default
The `defaultAnnot` function sets the annotation SQLite database back to the
original version provided by `customCMPdb`. This is achieved by deleting the
existing (e.g. custom) database and re-downloading a fresh instance from 
`AnnotationHub`.
```{r default, eval=FALSE}
defaultAnnot()
```

# Query Annotation Database
The `queryAnnotDB` function can be used to query the compound annotations from
the default resources as well as the custom resources stored in the SQLite
annotation database. The query can be a set of ChEMBL IDs. In this case it
returns a `data.frame` containing the annotations of the matching compounds
from the selected annotation resources specified under the \code{annot}
argument. The `listAnnot` function returns the names that can be assigned to
the `annot` argument.
```{r query, eval=TRUE, message=FALSE}
query_id <- c("CHEMBL1064", "CHEMBL10", "CHEMBL113", "CHEMBL1004", "CHEMBL31574")
listAnnot()
qres <- queryAnnotDB(query_id, annot=c("drugAgeAnnot", "lincsAnnot"))
qres
# query the added custom annotation
addCustomAnnot(annot_tb, annot_name="myCustom")
qres2 <- queryAnnotDB(query_id, annot=c("lincsAnnot", "myCustom"))
qres2
```

Since the supported compound databases use different identifiers, a ChEMBL
ID mapping table is used to connect identical entries across databases as
well as to link out to other resources such as ChEMBL itself or PubChem. For
custom compounds, where ChEMBL IDs are not available yet, one can use
alternative and/or custom identifiers.
```{r not_chembl, eval=TRUE, message=FALSE}
query_id <- c("BRD-A00474148", "BRD-A00150179", "BRD-A00763758", "BRD-A00267231")
qres3 <- queryAnnotDB(chembl_id=query_id, annot=c("lincsAnnot"))
qres3
```

# Supplemental Material
## Description of Annotation Tables in SQLite Database
The DrugAge database is manually curated by experts. It contains an extensive 
compilation of drugs, compounds and supplements (including natural products and 
nutraceuticals) with anti-aging properties that extend longevity in model 
organisms [@Barardo2017-xk]. The DrugAge build2 database was downloaded from
[here](https://genomics.senescence.info/drugs/dataset.zip) as a CSV file. The 
downloaded `drugage.csv` file contains `compound_name`, `synonyms`, `species`, `strain`,
`dosage`, `avg_lifespan_change`, `max_lifespan_change`, `gender`, `significance`,
and `pubmed_id` annotation columns. Since the DrugAge database only contains the
drug name as identifiers, it is necessary to map the drug name to other uniform
drug identifiers, such as ChEMBL IDs. In this package,
the drug names have been mapped to [ChEMBL](https://www.ebi.ac.uk/chembl/) [@Gaulton2012-ji],
[PubChem](https://pubchem.ncbi.nlm.nih.gov/) [@Kim2019-tg] and DrugBank IDs semi-manually
and stored under the `inst/extdata` directory named as `drugage_id_mapping.tsv`. 
Part of the id mappings in the `drugage_id_mapping.tsv` table is generated 
by the \code{processDrugage} function for compound names that have ChEMBL 
ids from the ChEMBL database (version 24). The missing IDs were added 
manually. A semi-manual approach was to use this 
[web service](https://cts.fiehnlab.ucdavis.edu/batch). After the semi-manual process,
the left ones were manually mapped to ChEMBL, PubChem and DrugBank ids. The 
entries that are mixture like green tee extract or peptide like Bacitracin were commented.
Then the `drugage_id_mapping` table was built into the annotation SQLite database
named as `compoundCollection_0.1.db` with table name of `drugAgeAnnot` by `buildDrugAgeDB` function.

The DrugBank annotation table (`DrugBankAnnot`) was downloaded from the DrugBank database
in [xml file](https://www.drugbank.ca/releases/latest).
The most recent release version at the time of writing this document is 5.1.5.  
The extracted xml file was processed by the \code{dbxml2df} function in this package.
`dbxml2df` and `df2SQLite` functions in this package were used to load the xml 
file into R and covert to a data.frame R object, then stored in the 
`compoundCollection` SQLite annotation database.
There are 55 annotation columns in the DrugBank annotation table, such as
`drugbank_id`, `name`, `description`, `cas-number`, `groups`, `indication`, 
`pharmacodynamics`, `mechanism-of-action`, `toxicity`, `metabolism`, `half-life`, 
`protein-binding`, `classification`, `synonyms`, `international-brands`, `packagers`, 
`manufacturers`, `prices`, `dosages`, `atc-codes`, `fda-label`, `pathways`, `targets`. 
The DrugBank id to ChEMBL id mappings were obtained from 
[UniChem](ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src2.txt.gz).

The CMAP02 annotation table (`cmapAnnot`) was processed from the downloaded compound 
[instance table](http://www.broadinstitute.org/cmap/cmap_instances_02.xls)
using the `buildCMAPdb` function defined by this package. The CMAP02 instance table contains
the following drug annotation columns: `instance_id`, `batch_id`, `cmap_name`, `INN1`,
`concentration (M)`, `duration (h)`, `cell2`, `array3`, `perturbation_scan_id`, 
`vehicle_scan_id4`, `scanner`, `vehicle`, `vendor`, `catalog_number`, `catalog_name`. 
Drug names are used as drug identifies. The `buildCMAPdb` function maps the drug 
names to external drug ids including `UniProt` [@The_UniProt_Consortium2017-bx], 
`PubChem`, `DrugBank` and `ChemBank` [@Seiler2008-dw] ids. It also adds additional
annotation columns such as `directionality`, `ATC codes` and `SMILES structure`.
The generated `cmap.db` SQLite database from `buildCMAPdb` function contains both
compound annotation table and structure information. The ChEMBL id mappings were
further added to the annotation table via PubChem CID to ChEMBL id mappings from 
[UniChem](ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src22.txt.gz).
The CMAP02 annotation table was stored in the `compoundCollection` SQLite annotation
database. Then the CMAP internal IDs to ChEMBL id mappings were added to the ID 
mapping table. 

The LINCS 2017 compound annotation table (`lincsAnnot`) was downloaded from 
[GEO](ftp://ftp.ncbi.nlm.nih.gov/geo/series/GSE92nnn/GSE92742/suppl/GSE92742_Broad_LINCS_pert_info.txt.gz)
where only compounds were selected. The annotation columns are `lincs_id`, `pert_name`,
`pert_type`, `is_touchstone`, `inchi_key_prefix`, `inchi_key`, `canonical_smiles`, `pubchem_cid`.
The annotation table was stored in the `compoundCollection` SQLite annotation database. 
Since the annotation only contains LINCS id to PubChem CID mapping, the LINCS ids 
were also mapped to ChEMBL ids via inchi key.

The SQLite annotation database also contains compound annotation tables of
`drugAge4` and `lincs2`. They contain annotations for compounds
in DrugAge build 4 database, and the newest LINCS beta database released in 2020
at [CLUE](https://clue.io/releases/data-dashboard).

# Session Info
```{r sessionInfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drugage_annot.R
\name{processDrugage}
\alias{processDrugage}
\title{Process Source DrugAge Dataset}
\usage{
processDrugage(dest_file = "drugage_id_mapping.tsv", verbose = TRUE)
}
\arguments{
\item{dest_file}{character(1), file path to the generated DrugAge annotation 
tabular file with id mappings. The default will write the file named as
"drugage_id_mapping.tsv" to user's current working directory.}

\item{verbose}{logical(1), If descriptive message and list of issues should be 
included as output}
}
\value{
write the default 'drugage_id_mapping.tsv' file to user's current working
directory or the file path defined by users to the \code{dest_dafile} argument.
}
\description{
This function processes the source DrugAge datasets by adding the ChEMBL, 
PubChem and DrugBank id mapping information to the source DrugAge table 
which only has compound names without id mapping information. Source file of 
DrugAge is linked here: \url{http://genomics.senescence.info/drugs/dataset.zip}
}
\details{
This function only annotates compound names that have ChEMBL 
ids from the ChEMBL database (version 24). The missing IDs were added 
manually. A semi-manual approach was to use this web service: 
\url{https://cts.fiehnlab.ucdavis.edu/batch}. After the semi-manual process,
the left ones were manually mapped to ChEMBL, PubChem and DrugBank ids. 
The mixed items were commented.
}
\examples{
library(ChemmineR)
\dontrun{
processDrugage(dest_file="drugage_id_mapping.tsv")
# Now the missing IDs need to be added manually. A semi-manual approach is to 
# use this web service: https://cts.fiehnlab.ucdavis.edu/batch
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drugage_annot.R
\name{buildDrugAgeDB}
\alias{buildDrugAgeDB}
\title{Build DrugAge Annotation Database}
\usage{
buildDrugAgeDB(
  da_path = system.file("extdata/drugage_id_mapping.tsv", package = "customCMPdb"),
  dest_path
)
}
\arguments{
\item{da_path}{character(1), file path to the tabular file generated from
\code{processDrugage} function and manually edited the missing IDs.}

\item{dest_path}{character(1), destination path of the result DrugAge 
annotation SQLite database}
}
\value{
DrugAge annotation SQLite database
}
\description{
This function builds the DrugAge annotation SQLite database from the 
'drugage_id_mapping' table stored in the 'inst/extdata' directory of this 
package. The 'drugage_id_mapping.tsv' table contains the DrugAge compounds 
annotation information (such as species, avg_lifespan_change etc) as well 
as the compound name to ChEMBL id and PubChem id mappings.
}
\details{
Part of the id mappings in the 'drugage_id_mapping.tsv' table is generated 
by the \code{processDrugage} function for compound names that have ChEMBL 
ids from the ChEMBL database (version 24). The missing IDs were added 
manually. A semi-manual approach was to use this web service: 
\url{https://cts.fiehnlab.ucdavis.edu/batch}. After the semi-manual process,
the left ones were manually mapped to ChEMBL, PubChem and DrugBank ids. 
The mixed items were commented.
}
\examples{
buildDrugAgeDB(dest_path=tempfile(fileext="_drugage.db"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_fcts.R
\name{queryAnnotDB}
\alias{queryAnnotDB}
\title{Annotation Queries with Compound IDs}
\usage{
queryAnnotDB(
  chembl_id,
  annot = c("drugAgeAnnot", "DrugBankAnnot", "cmapAnnot", "lincsAnnot")
)
}
\arguments{
\item{chembl_id}{character vector of ChEMBL IDs or compound ids from other 
annotation system..}

\item{annot}{character vector of annotation resources, such as
\code{drugAgeAnnot}, \code{DrugBankAnnot}, \code{cmapAnnot}, \code{lincsAnnot} 
or names of the annotation tables added by users. The \code{\link{listAnnot}}
function lists the available options for the \code{annot} argument names.}
}
\value{
data.frame of annotation result
}
\description{
This function can be used to query compound annotations from the default
resources as well as the custom resources stored in the SQLite annotation
database. The default annotation resources are DrugAge, DrugBank, CMAP02 and
LINCS. The customized compound annotations could be added/deleted by the
\code{\link{customAnnot}} utilities.
}
\details{
The input of this query function could be a set of ChEMBL IDs, it returns a
data.frame storing annotations of the input compounds from selected
annotation resources defined by the \code{annot} argument.

Since in the SQLite annotation database, ID identifiers from different ID systems, 
such as DrugBank and LINCS, are connected by ChEMBL IDs, it is hard to tell 
whether two IDs, such as DB00341, BRD-A42571354, refer to the same compound if 
either of them lack ID mappings to ChEMBL. So for querying compounds that don't 
have ChEMBL IDs, only one isolated database where the compounds belong to are 
supported. For example, a compound with LINCS id as "BRD-A00150179" doesn't have 
the ChEMBL ID mapping, when it is passed to the `chembl_id` argument, 
the `annot` need only to be set as `lincsAnnot` and the result
will be the compound annotation table from the LINCS annotation.
}
\examples{
query_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL100109",
               "CHEMBL100", "CHEMBL1000", "CHEMBL10")
qres <- queryAnnotDB(query_id, annot=c("drugAgeAnnot", "lincsAnnot"))

# Add a custom compound annotation table
chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
               "CHEMBL100", "CHEMBL1000", NA)
annot_tb <- data.frame(compound_name=paste0("name", 1:6),
        chembl_id=chembl_id,
        feature1=paste0("f", 1:6),
        feature2=rnorm(6))
addCustomAnnot(annot_tb, annot_name="myCustom2")

# query custom annotation
qres2 <- queryAnnotDB(query_id, annot=c("lincsAnnot", "myCustom2"))

# query compounds that don't have ChEMBL IDs
query_id <- c("BRD-A00474148", "BRD-A00150179", "BRD-A00763758", "BRD-A00267231")
qres3 <- queryAnnotDB(chembl_id=query_id, annot=c("lincsAnnot"))
qres3
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbxml_handle.R
\name{df2SQLite}
\alias{df2SQLite}
\title{Store drugbank dataframe into an SQLite database}
\usage{
df2SQLite(dbdf, version, dest_dir = ".")
}
\arguments{
\item{dbdf}{Drugbank dataframe generated by \code{\link{dbxml2df}} function.}

\item{version}{Character(1), version of the input drugbank dataframe generated 
by \code{\link{dbxml2df}} function}

\item{dest_dir}{Character(1), destination directory that the result SQLite 
database stored in. The default is user's current working directory}
}
\value{
SQLite database named as "drugbank_<versionNumber>.db" stored under user's 
present working directory of R session or user's specified directory.
}
\description{
Store specific version of drugbank dataframe into an SQLite database 
under user defined directory, the default is user's present working directory 
of R session
}
\examples{
library(RSQLite)
\dontrun{
# download the original drugbank database (http://www.drugbank.ca/releases/latest) (xml file) 
# to your current R working directory, and rename as "drugbank.xml".
# Read in the xml file and convert to a data.frame in R 

drugbank_df = dbxml2df(xmlfile="drugbank.xml", version="5.1.5")

# store the converted drugbank dataframe into SQLite database under user's
present R working direcotry, or other directory defined by 'dest_dir'

df2SQLite(dbdf=drugbank_df, version="5.1.5") # set version as version of xml file
}
}
\seealso{
\code{\link{dbxml2df}}
}
\author{
Yuzhu Duan \url{yduan004@ucr.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manageCMAP.R
\name{buildCMAPdb}
\alias{buildCMAPdb}
\title{Build CMAP Database}
\usage{
buildCMAPdb(dest_dir = ".")
}
\arguments{
\item{dest_dir}{character(1), destination directory under which the result 
SQLite database named as 'cmap.db' stored. The default is user's current
working directory.}
}
\value{
write "cmap.db" SQLite database to the destination directory defined by user.
}
\description{
This function builds a SQLite database named as 'cmap.db' that contains id 
mappings of cmap names to PubChem/DrugBank IDs as well as compound structure information.
}
\details{
For about 2/3 of the CMAP drugs, one can obtain their PubChem/DrugBank IDs from 
the DMAP site here: http://bio.informatics.iupui.edu/cmaps. Since this website is no
longer supported, the processed CMAP name to PubChem and DrugBank ID mapping table
is stored under the "inst/extdata" folder of this package named as "dmap_unique.txt".  
The SMILES strings for CMAP entries were obtained from ChemBank. Compounds 
were matched by names using the 'stringdist' library where cmap_name from 
CMAP were mapped to the closest name in ChemBank.
}
\examples{
library(ChemmineR)
## Query database
# buildCMAPdb(dest_dir="./inst/scripts")
# conn <- initDb("/inst/scripts/cmap.db")
# results <- getAllCompoundIds(conn)
# sdfset <- getCompounds(conn, results, keepOrder=TRUE)
# sdfset
# as.data.frame(datablock2ma(datablock(sdfset)))[1:4,]
# myfeat <- listFeatures(conn)
# feat <- getCompoundFeatures(conn, results, myfeat)
# feat[1:4,]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/customCMPdb-package.R
\docType{package}
\name{customCMPdb-package}
\alias{customCMPdb-package}
\alias{customCMPdb}
\title{Customize and Query Compound Annotation Database}
\description{
This package is served as the query and customization interface for compound 
annotations from \href{https://genomics.senescence.info/drugs/}{DrugAge}, 
\href{https://www.drugbank.ca/}{DrugBank}, 
\href{https://portals.broadinstitute.org/cmap/}{CMAP02}
and \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742}{LINCS} databases.
It also stores the structure SDF datasets for compounds in the above four databases.

Specifically, the annotation database created by this package is an SQLite database
containing 5 tables, including 4 compound annotation tables from DrugAge, 
DrugBank, CMAP02 and LINCS databases, respectively. The other one is an ID 
mapping table of ChEMBL IDs to IDs of individual databases. The other 4 datasets 
stores the structures of compounds in the DrugAge, DrugBank, CMAP02 and LINCS 
databases in SDF files. For detailed description of the 5 datasets generated
by this package, please consult to the vignette of this package by running
\code{browseVignettes("customCMPdb")}. The actual datasets are hosted in 
\code{AnnotationHub}.

This package also provides functionalities to customize and query the compound
annotation SQLite database. Users could add their customized compound annotation
tables to the SQLite database and query both the default (DrugAge, DrugBank, CMAP02,
LINCS) and customized annotations by providing ChEMBL ids of the query compounds.
The customization and query functions are available at \code{\link{customAnnot}}
and \code{\link{queryAnnotDB}}, respectively.
}
\details{
The description of the 5 datasets in this package is as follows.

\strong{Annotation SQLite database:}

It is a SQLite database storing compound annotation tables for DrugAge, 
DrugBank, CMAP02 and LINCS, respectively. It also contains an ID mapping 
table of ChEMBL ID to IDs of individual databases. 

\strong{DrugAge SDF:}

It is an SDF (Structure-Data File) file storing molecular structures of 
DrugAge compounds. The source DrugAge annotation file was downloaded from
\href{http://genomics.senescence.info/drugs/dataset.zip}{here}. The extracted csv 
file only contains drug names, without id mappings to external resources 
such as PubChem or ChEMBL. The extracted 'drugage.csv' file was further processed by the 
\code{\link{processDrugage}} function in this package. The result DrugAge annotation table
as well as the id-mapping table (DrugAge internal id to ChEMBL ID) were then
stored in the SQLite annotation database named as 'compoundCollection'. 
The drug structures were obtained from PubChem CIDs by \code{\link[ChemmineR]{getIds}} 
function from \pkg{ChemmineR} package. The \code{SDFset} object was then 
written to the \code{drugage_build2.sdf} file

\strong{DrugBank SDF:}

This SDF file stores structures of compounds in 
\href{https://www.drugbank.ca/}{DrugBank} database. The full DrugBank xml 
file was downloaded from \url{https://www.drugbank.ca/releases/latest}.
The most recent release version at the time of writing this document is 5.1.5.  
The extracted xml file was processed by the \code{\link{dbxml2df}} function in this package.
The result DrugBank annotation table was then stored in the \code{compoundCollection}
SQLite database. The DrugBank to ChEMBL id mappings were obtained from 
\href{ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src2.txt.gz}{UniChem}.
The DrugBank SDF file was downloaded from 
\url{https://www.drugbank.ca/releases/latest#structures}.
Some validity checks and modifications were made via utilities in the 
\pkg{ChemmineR} package. The results were written to the \code{drugbank_5.1.5.sdf} file

\strong{CMAP SDF:}

The CMAP compound instance table was downloaded from 
\href{http://www.broadinstitute.org/cmap/cmap_instances_02.xls}{CMAP02} 
website and processed by the \code{\link{buildCMAPdb}} function
in this package. The result 'cmap.db' contains both compound annotation and 
structure information. 
Since the annotation table only contains PubChem CID, the ChEMBL ids were added 
via PubChem CID to ChEMBL id mappings from 
\href{ftp://ftp.ebi.ac.uk/pub/databases/chembl/UniChem/data/wholeSourceMapping/src_id1/src1src22.txt.gz}{UniChem}.
The CMAP internal IDs were made for ChEMBL id to CMAP id mappings. The 
structures were written to the \code{cmap02.sdf} file

\strong{LINCS SDF:}

The LINCS compound annotation table was downloaded from
\href{ftp://ftp.ncbi.nlm.nih.gov/geo/series/GSE92nnn/GSE92742/suppl/GSE92742_Broad_LINCS_pert_info.txt.gz}{GEO}.
where only compounds type were selected.
The LINCS ids were mapped to ChEMBL ids via inchi key. The LINCS compounds 
structures were obtained from PubChem CIDs via \code{\link[ChemmineR]{getIds}} function from
\pkg{ChemmineR} package. The structures were written to the \code{lincs_pilot1.sdf} file

The R script of generating the above 5 datasets is available at the 
'inst/scripts/make-data.R' file in this package.  The file location can
be found by running \code{system.file("scripts/make-data.R",package="customCMPdb")}
in user's R session or from the 
\href{https://github.com/yduan004/customCMPdb/blob/master/inst/scripts/make-data.R}{GitHub repository}
of this package.
}
\examples{
library(AnnotationHub)
\dontrun{
    ah <- AnnotationHub()
    
    ## Load compoundCollection annotation SQLite database
    query(ah, c("customCMPdb", "annot_0.1"))
    annot_path <- ah[["AH79563"]]
    library(RSQLite)
    conn <- dbConnect(SQLite(), annot_path)
    dbListTables(conn)
    drugAgeAnnot <- dbReadTable(conn, "drugAgeAnnot")
    head(drugAgeAnnot)
    dbDisconnect(conn)
    
    ## Load DrugAge SDF file
    query(ah, c("customCMPdb", "drugage_build2"))
    da_path <- ah[["AH79564"]]
    da_sdfset <- ChemmineR::read.SDFset(da_path)
    
    ## Load DrugBank SDF file
    query(ah, c("customCMPdb", "drugbank_5.1.5"))
    db_path <- ah[["AH79565"]]
    db_sdfset <- ChemmineR::read.SDFset(db_path)
    
    ## Load CMAP SDF file
    query(ah, c("customCMPdb", "cmap02"))
    cmap_path <- ah[["AH79566"]]
    cmap_sdfset <- ChemmineR::read.SDFset(cmap_path)
    
    ## Load LINCS SDF file
    query(ah, c("customCMPdb", "lincs_pilot1"))
    lincs_path <- ah[["AH79567"]]
    lincs_sdfset <- ChemmineR::read.SDFset(lincs_path)
}
}
\seealso{
\code{\link{customAnnot}}, \code{\link{queryAnnotDB}}
}
\author{
\itemize{
  \item Yuzhu Duan (yduan004@ucr.edu)
  \item Thomas Girke (thomas.girke@ucr.edu)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/customAnnot.R
\name{addCustomAnnot}
\alias{addCustomAnnot}
\alias{customAnnot}
\alias{deleteAnnot}
\alias{listAnnot}
\alias{defaultAnnot}
\title{Add/Delete Custom Annotation}
\usage{
addCustomAnnot(annot_tb, id_col = NULL, annot_name, overwrite = FALSE)

deleteAnnot(annot_name)

listAnnot()

defaultAnnot()
}
\arguments{
\item{annot_tb}{data.frame representing the custom annotation table,
Note, it should contains a 'chembl_id' column representing the compound
ChEMBL ids}

\item{id_col}{column name in \code{annot_tb} that is used as ID column, 
this column must contain unique identifiers. If not defined, an automatically
generated ID column will be appended.}

\item{annot_name}{character(1), user defined name of the annotation table}

\item{overwrite}{a logical specifying whether to overwrite an existing table 
or not. Its default is FALSE.}
}
\value{
character vector of names of the annotation tables in the SQLite DB

character(1), path to the annotation SQLite database
}
\description{
Functions could be used to add/delete user's custom compound annotations
from the annotation SQLite database.
The added custom compound annotation table should contains a column named as
\code{chembl_id} that represents the ChEMBL ids of the added compounds.

The \code{listAnnot} function lists the available annotation
resources in the SQLite annotation database.

The \code{defaultAnnot} function sets the annotation SQLite
database to the default one by deleting the existing one and re-downloading
from AnnotationHub.
}
\examples{
chembl_id <- c("CHEMBL1000309", "CHEMBL100014", "CHEMBL10",
               "CHEMBL100", "CHEMBL1000", NA)
annot_tb <- data.frame(compound_name=paste0("name", 1:6),
        chembl_id=chembl_id,
        feature1=paste0("f", 1:6),
        feature2=rnorm(6))
addCustomAnnot(annot_tb, annot_name="mycustom3")
deleteAnnot("mycustom3")
annot_names <- listAnnot()
# defaultAnnot()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/customAnnot.R
\name{loadAnnot}
\alias{loadAnnot}
\title{Load Compound Annotation Database}
\usage{
loadAnnot()
}
\description{
The compound annotation tables from different databases/sources are stored
in one SQLite database. This function can be used to load the SQLite 
annotation database
}
\examples{
conn <- loadAnnot()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbxml_handle.R
\name{dbxml2df}
\alias{dbxml2df}
\title{Convert drugbank database (xml file) into dataframe.}
\usage{
dbxml2df(xmlfile, version)
}
\arguments{
\item{xmlfile}{Character(1), file path to the xml file downloaded from the
DrugBank website at \url{https://www.drugbank.ca/releases/latest}}

\item{version}{Character(1), DrugBank version of the xml file}
}
\value{
Dataframe of drugbank xml database.
}
\description{
Download the original DrugBank database 
at \url{http://www.drugbank.ca/releases/latest} (xml file) into your current 
working directory and rename as "drugbank.xml"
then run: 
\code{drugbank_df = dbxml2df(xmlfile="drugbank.xml", version="5.0.10")}.
}
\note{
This process with take about 20 minutes.
}
\examples{
library(XML)
\dontrun{
## download the original drugbank database at 
\url{http://www.drugbank.ca/releases/latest} (xml file)
## into your current directory and rename as drugbank.xml

## convert drugbank database (xml file) into dataframe: 
drugbank_df <- dbxml2df(xmlfile="drugbank.xml", version="5.0.10")
}
}
\references{
\url{http://www.drugbank.ca/releases/latest}
}
\seealso{
\code{\link{df2SQLite}}
}
\author{
Yuzhu Duan \url{yduan004@ucr.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{loadSDFwithName}
\alias{loadSDFwithName}
\title{Load Compound Structures from Four Resources}
\usage{
loadSDFwithName(source = "LINCS")
}
\arguments{
\item{source}{character(1), one of "CMAP2", "LINCS", "DrugBank", "DrugAge"}
}
\value{
SDFset object of compounds in the \code{source} database, the \code{cid}
of the SDFset are compound names.
}
\description{
This function could be used to get SDFset of compounds in CMAP2, LINCS 2017, 
DrugAge build 2 or DrugBank 5.1.5 databases. The \code{cid} of the SDFset are 
compound names instead of their internal IDs.
}
\examples{
da_sdf <- loadSDFwithName(source="DrugAge")
}
\seealso{
\code{\link[ChemmineR]{SDFset}}
}
